@@ -24,6 +24,8 @@
  * Change Log
  *      28-Nov-2003 Lineo Solutions, Inc.   Add module parameter. 'shortpacket'
  *                                          and 'recvpacket'.
+ *      26-Feb-2004 Lineo Solutions, Inc.  for Tosa (support cable gpio)
+ *
  *
  */
 
@@ -88,7 +90,7 @@ USBD_MODULE_INFO ("pxa_bi 0.1-alpha");
 #include "pxa.h"
 
 
-#if 1   /* kabata add start 2003/10/08 */
+#ifdef CONFIG_ARCH_SHARP_SL   /* kabata add start 2003/10/08 */
 static int  shortpacket = 0;
 MODULE_PARM(shortpacket, "i");
 
@@ -484,7 +486,7 @@ static void pxa_out_flush(int ep) {
 static __inline__ void pxa_out_n(int ep, struct usb_endpoint_instance *endpoint)
 {
         if (UDCCSN(ep) & UDCCS_BO_RPC) {
-#if 1   /* kabata add start 2003/10/08 */
+#ifdef CONFIG_ARCH_SHARP_SL   /* kabata add start 2003/10/08 */
                 // clear RPC and interrupt
                 UDCCSN(ep) = UDCCS_BO_RPC;
                 pxa_ep_reset_irs(ep);
@@ -519,7 +521,7 @@ static __inline__ void pxa_out_n(int ep, struct usb_endpoint_instance *endpoint)
                         }
                 }
         }
-#if 1   /* kabata chg start 2003/10/08 */
+#ifdef CONFIG_ARCH_SHARP_SL   /* kabata chg start 2003/10/08 */
         else {
                 // clear interrupt
                 pxa_ep_reset_irs(ep);
@@ -606,7 +608,7 @@ static void __inline__ pxa_start_n (unsigned int ep, struct usb_endpoint_instanc
                                 UDDRN(ep) = *cp++;
                         }
 
-#if 0   /* kabata chg start 2003/10/08 */
+#ifndef CONFIG_ARCH_SHARP_SL   /* kabata chg start 2003/10/08 */
                         if (( last < endpoint->tx_packetSize ) ||
                                         ( (endpoint->tx_urb->actual_length - endpoint->sent ) == last )) 
 #else
@@ -912,6 +914,10 @@ static __inline__ void pxa_ep0(struct usb_endpoint_instance *endpoint, volatile
         PXA_REGS(UDCCS0,"  <-- ep0");
 }
 
+#ifdef USBD_CABLE_GPIO
+static int udc_connect_state = 0;
+#endif
+
 /* ********************************************************************************************* */
 /* Interrupt Handler
  */
@@ -975,10 +981,24 @@ static void int_hndlr (int irq, void *dev_id, struct pt_regs *regs)
         // uncommon interrupts
         if ((udccr = UDCCR) & (UDCCR_RSTIR | UDCCR_RESIR | UDCCR_SUSIR)) {
 
+#ifdef USBD_CABLE_GPIO
+		if (!udc_connect_state && (udccr & UDCCR_RSTIR)) {
+		  //	    printk("ignore UDC Reset\n");
+                        UDCCR |= UDCCR_RSTIR;
+			udccr &= ~UDCCR_RSTIR;
+		}
+		if (!udc_connect_state && (udccr & UDCCR_RESIR)) {
+		  //	    printk("ignore UDC Resume\n");
+                        UDCCR |= UDCCR_RESIR;
+			udccr &= ~UDCCR_RESIR;
+		}
+#endif
+
                 // UDC Reset
-                if (udccr & UDCCR_RSTIR) {
+		if (udccr & UDCCR_RSTIR) {
 #if defined(CONFIG_ARCH_SHARP_SL) && defined(CONFIG_APM_CPU_IDLE)
 			static int is1stReset = 1;
+			//	    printk("UDC Reset\n");
 			if (is1stReset) {
 				is1stReset = 0;
 			} else {
@@ -995,7 +1015,8 @@ static void int_hndlr (int irq, void *dev_id, struct pt_regs *regs)
                 }
 
                 // UDC Resume
-                if (udccr & UDCCR_RESIR) {
+		if (udccr & UDCCR_RESIR) {
+		  //	    printk("UDC Resume\n");
                         PXA_CCR(udccr, "------------------------> Resume");
                         if (udc_suspended) {
 #if defined(CONFIG_ARCH_SHARP_SL) && defined(CONFIG_APM_CPU_IDLE)
@@ -1009,6 +1030,7 @@ static void int_hndlr (int irq, void *dev_id, struct pt_regs *regs)
 
                 // UDC Suspend
                 if (udccr & UDCCR_SUSIR) {
+		  //	    printk("UDC Suspend\n");
                         PXA_CCR(udccr, "------------------------> Suspend");
                         if (!udc_suspended) {
 #if defined(CONFIG_ARCH_SHARP_SL) && defined(CONFIG_APM_CPU_IDLE)
@@ -1195,7 +1217,7 @@ void udc_setup_ep (struct usb_device_instance *device, unsigned int ep,
 		else if (endpoint->endpoint_address) {
 
 
-#if 0   /* kabata chg start 2003/10/08 */
+#ifndef CONFIG_ARCH_SHARP_SL   /* kabata chg start 2003/10/08 */
 			usbd_fill_rcv (device, endpoint, 5);
 #else
 			usbd_fill_rcv (device, endpoint, recvpacket);
@@ -1284,7 +1306,17 @@ void udc_disable_ep (unsigned int ep)
  */
 int udc_connected ()
 {
-	return 1;
+	int rc = 1;
+#ifdef USBD_CABLE_GPIO
+#ifdef USBD_CABLE_ACTIVE_HIGH
+        rc = (GPLR(USBD_CABLE_GPIO) & GPIO_bit(USBD_CABLE_GPIO)) != 0;
+        //dbg_udc (1, "udc_connected: ACTIVE_HIGH: %d", rc);
+#else
+        rc = (GPLR(USBD_CABLE_GPIO) & GPIO_bit(USBD_CABLE_GPIO)) == 0;
+        //dbg_udc (1, "udc_connected: ACTIVE_LOW: %d", rc);
+#endif
+#endif
+	return rc;
 }
 
 /**
@@ -1294,6 +1326,9 @@ int udc_connected ()
  */
 void udc_connect (void)
 {
+#ifdef USBD_CABLE_GPIO
+  udc_connect_state = 1;
+#endif
 #ifdef CONFIG_SABINAL_DISCOVERY
         /* Set GPIO pin function to I/O */
         GAFR1_U &= ~(0x3 << ((USBD_CONNECT_GPIO & 0xF) << 1));
@@ -1318,6 +1353,20 @@ void udc_connect (void)
 	GPCR(USBD_CONNECT_GPIO) = GPIO_bit(USBD_CONNECT_GPIO);
 #endif
 
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+	if (!udc_connected()) {
+		//printk("USB Cable not connected!!\n");
+		return;
+	}
+	SCP2_REG_GPCR |= USBD_CONNECT_GPIO;
+#if defined(USBD_CONNECT_HIGH)
+	/* Set GPIO pin high to connect */
+	set_scoop2_gpio(USBD_CONNECT_GPIO);
+#else
+	/* Set GPIO pin low to connect */
+	reset_scoop2_gpio(USBD_CONNECT_GPIO);
+#endif
+
 #else
 #warning NO USB Device connect
 #endif
@@ -1330,6 +1379,9 @@ void udc_connect (void)
  */
 void udc_disconnect (void)
 {
+#ifdef USBD_CABLE_GPIO
+  udc_connect_state = 0;
+#endif
 #ifdef CONFIG_SABINAL_DISCOVERY
         /* Set GPIO pin function to I/O */
         GAFR1_U &= ~(0x3 << ((USBD_CONNECT_GPIO & 0xF) << 1));
@@ -1354,18 +1406,35 @@ void udc_disconnect (void)
 	GPSR(USBD_CONNECT_GPIO) = GPIO_bit(USBD_CONNECT_GPIO);
 #endif
 
+#elif defined(CONFIG_ARCH_PXA_TOSA)
+	SCP2_REG_GPCR |= USBD_CONNECT_GPIO;
+#if defined(USBD_CONNECT_HIGH)
+	/* Set GPIO pin low to disconnect */
+	reset_scoop2_gpio(USBD_CONNECT_GPIO);
+#else
+	/* Set GPIO pin high to disconnect */
+	set_scoop2_gpio(USBD_CONNECT_GPIO);
+#endif
+
 #else
 #warning NO USB Device connect
 #endif
 }
 
-#if 0
+#ifdef USBD_CABLE_GPIO
 /**
  * udc_int_hndlr_cable - interrupt handler for cable
  */
 static void udc_int_hndlr_cable (int irq, void *dev_id, struct pt_regs *regs)
 {
 	// GPIOn interrupt
+	//dbg_udc("udc_cradle_interrupt: ->%08x\n",GPLR(USBD_CABLE_GPIO));
+  if (udc_connected()) {
+    usbd_device_event(udc_device, DEVICE_HUB_RESET, 0);
+    udc_cable_event ();
+  } else {
+    udc_disconnect();
+  }
 }
 #endif
 
@@ -1663,12 +1732,14 @@ int udc_request_udc_irq ()
  */
 int udc_request_cable_irq ()
 {
-#ifdef XXXX_CABLE_IRQ
+#ifdef USBD_CABLE_GPIO
+	set_GPIO_mode(USBD_CABLE_GPIO | GPIO_IN);
+	set_GPIO_IRQ_edge(USBD_CABLE_GPIO, GPIO_BOTH_EDGES);
 	// request IRQ  and IO region
-	if (request_irq
-	    (XXXX_CABLE_IRQ, int_hndlr, SA_INTERRUPT | SA_SAMPLE_RANDOM, UDC_NAME " Cable",
-	     NULL) != 0) {
-		printk (KERN_INFO "usb_ctl: Couldn't request USB irq\n");
+	if (request_irq (IRQ_GPIO(USBD_CABLE_GPIO), udc_int_hndlr_cable,
+		SA_INTERRUPT | SA_SAMPLE_RANDOM, UDC_NAME " Cable", NULL) != 0)
+	{
+		printk (KERN_INFO "usb_ctl: Couldn't request USB cable irq\n");
 		return -EINVAL;
 	}
 #endif
@@ -1921,8 +1992,8 @@ void udc_release_udc_irq ()
  */
 void udc_release_cable_irq ()
 {
-#ifdef XXXX_IRQ
-	free_irq (XXXX_CABLE_IRQ, NULL);
+#ifdef USBD_CABLE_GPIO
+	free_irq (IRQ_GPIO(USBD_CABLE_GPIO), NULL);
 #endif
 }
 
