@@ -12,6 +12,8 @@
  * Change Log
  *     17-Sep-2002 Lineo Japan, Inc.
  *     13-Mar-2003 Sharp for Shepherd
+ *     26-Feb-2004 Lineo Solutions, Inc.  for Tosa
+ *     29-Sep-2004 Lineo Solutions, Inc.  for Spitz
  */
 
 #include <linux/config.h>
@@ -19,24 +21,19 @@
 
 static inline void arch_idle(void)
 {
-	if (!hlt_counter) {
-		int flags;
-		local_irq_save(flags);
-		if(!current->need_resched)
-			cpu_do_idle(0);
-		local_irq_restore(flags);
-	}
+	cpu_do_idle();
 }
 
-
 static inline void arch_reset(char mode)
 {
 #ifdef CONFIG_ARCH_SHARP_SL
-#ifdef CONFIG_ARCH_PXA_SHEPHERD
+#ifdef CONFIG_PM
+#if defined(CONFIG_ARCH_PXA_SHEPHERD) || defined(CONFIG_ARCH_PXA_TOSA)
 	sharpsl_restart_nonstop();
 #else
 	sharpsl_restart();
 #endif
+#endif
 #else
 	if (mode == 's') {
 		/* Jump into ROM at address 0 */
@@ -48,7 +45,7 @@ static inline void arch_reset(char mode)
 #endif
 		OWER = OWER_WME;
 		OSSR = OSSR_M3;
-		OSMR3 = OSCR + 36864;	/* ... in 10 ms */
+		OSMR3 = OSCR + LATCH * 10;	/* ... in 100 ms */
 	}
 #endif
 }
