@@ -102,6 +102,8 @@ static struct timer_vec * const tvecs[] = {
 	(struct timer_vec *)&tv1, &tv2, &tv3, &tv4, &tv5
 };
 
+static struct list_head * run_timer_list_running;
+
 #define NOOF_TVECS (sizeof(tvecs) / sizeof(tvecs[0]))
 
 void init_timervecs (void)
@@ -129,7 +131,9 @@ static inline void internal_add_timer(struct timer_list *timer)
 	unsigned long idx = expires - timer_jiffies;
 	struct list_head * vec;
 
-	if (idx < TVR_SIZE) {
+	if (run_timer_list_running)
+		vec = run_timer_list_running;
+	else if (idx < TVR_SIZE) {
 		int i = expires & TVR_MASK;
 		vec = tv1.vec + i;
 	} else if (idx < 1 << (TVR_BITS + TVN_BITS)) {
@@ -290,6 +294,7 @@ static inline void run_timer_list(void)
 {
 	spin_lock_irq(&timerlist_lock);
 	while ((long)(jiffies - timer_jiffies) >= 0) {
+		LIST_HEAD(queued);
 		struct list_head *head, *curr;
 		if (!tv1.index) {
 			int n = 1;
@@ -297,6 +302,7 @@ static inline void run_timer_list(void)
 				cascade_timers(tvecs[n]);
 			} while (tvecs[n]->index == 1 && ++n < NOOF_TVECS);
 		}
+		run_timer_list_running = &queued;
 repeat:
 		head = tv1.vec + tv1.index;
 		curr = head->next;
@@ -318,8 +324,18 @@ repeat:
 			timer_exit();
 			goto repeat;
 		}
+		run_timer_list_running = NULL;
 		++timer_jiffies; 
 		tv1.index = (tv1.index + 1) & TVR_MASK;
+
+		curr = queued.next;
+		while (curr != &queued) {
+			struct timer_list *timer;
+
+			timer = list_entry(curr, struct timer_list, list);
+			curr = curr->next;
+			internal_add_timer(timer);
+		}			
 	}
 	spin_unlock_irq(&timerlist_lock);
 }
@@ -586,7 +602,14 @@ void update_process_times(int user_tick)
 	if (p->pid) {
 		if (--p->counter <= 0) {
 			p->counter = 0;
-			p->need_resched = 1;
+			/*
+			 * SCHED_FIFO is priority preemption, so this is 
+			 * not the place to decide whether to reschedule a
+			 * SCHED_FIFO task or not - Bhavesh Davda
+			 */
+			if (p->policy != SCHED_FIFO) {
+				p->need_resched = 1;
+			}
 		}
 		if (p->nice > 0)
 			kstat.per_cpu_nice[cpu] += user_tick;
@@ -656,12 +679,14 @@ static inline void update_times(void)
 	 * need to save/restore the flags of the local CPU here. -arca
 	 */
 	write_lock_irq(&xtime_lock);
+	vxtime_lock();
 
 	ticks = jiffies - wall_jiffies;
 	if (ticks) {
 		wall_jiffies += ticks;
 		update_wall_time(ticks);
 	}
+	vxtime_unlock();
 	write_unlock_irq(&xtime_lock);
 	calc_load(ticks);
 }
@@ -716,10 +741,18 @@ asmlinkage unsigned long sys_alarm(unsigned int seconds)
  * The Alpha uses getxpid, getxuid, and getxgid instead.  Maybe this
  * should be moved into arch/i386 instead?
  */
- 
+
+/**
+ * sys_getpid - return the thread group id of the current process
+ *
+ * Note, despite the name, this returns the tgid not the pid.  The tgid and
+ * the pid are identical unless CLONE_THREAD was specified on clone() in
+ * which case the tgid is the same in all threads of the same group.
+ *
+ * This is SMP safe as current->tgid does not change.
+ */
 asmlinkage long sys_getpid(void)
 {
-	/* This is SMP safe - current->pid doesn't change */
 	return current->tgid;
 }
 
@@ -795,9 +828,21 @@ asmlinkage long sys_getegid(void)
 
 #endif
 
+#ifdef CONFIG_KMC_PARTNER_VIRTUAL_ICE
+int __kmc_sys_getpid_flag;
+
+void __kmc_sys_getpid(void)
+{
+	asm("nop");
+}
+#endif
+
 /* Thread ID - the internal kernel "pid" */
 asmlinkage long sys_gettid(void)
 {
+#ifdef CONFIG_KMC_PARTNER_VIRTUAL_ICE
+	if (__kmc_sys_getpid_flag) __kmc_sys_getpid();
+#endif
 	return current->pid;
 }
 
