@@ -49,6 +49,7 @@
 #include <linux/iobuf.h>
 #include <linux/highmem.h>
 #include <linux/pagemap.h>
+#include <linux/module.h>
 
 #include <asm/pgalloc.h>
 #include <asm/uaccess.h>
@@ -56,6 +57,7 @@
 
 unsigned long max_mapnr;
 unsigned long num_physpages;
+unsigned long num_mappedpages;
 void * high_memory;
 struct page *highmem_start_page;
 
@@ -122,11 +124,11 @@ static inline void free_one_pgd(pgd_t * dir)
 		return;
 	}
 	pmd = pmd_offset(dir, 0);
-	pgd_clear(dir);
 	for (j = 0; j < PTRS_PER_PMD ; j++) {
 		prefetchw(pmd+j+(PREFETCH_STRIDE/16));
 		free_one_pmd(pmd+j);
 	}
+	pgd_clear(dir);
 	pmd_free(pmd);
 }
 
@@ -278,8 +280,16 @@ cont_copy_pmd_range:	src_pmd++;
 out_unlock:
 	spin_unlock(&src->page_table_lock);
 out:
+#ifdef __arm__
+	memc_update_mm(src);
+	memc_update_mm(dst);
+#endif
 	return 0;
 nomem:
+#ifdef __arm__
+	memc_update_mm(src);
+	memc_update_mm(dst);
+#endif
 	return -ENOMEM;
 }
 
@@ -357,37 +367,17 @@ static inline int zap_pmd_range(mmu_gather_t *tlb, pgd_t * dir, unsigned long ad
 	return freed;
 }
 
-void unmap_page_range(mmu_gather_t *tlb, struct mm_struct *mm, unsigned long address, unsigned long end)
-{
-	int freed = 0;
-	pgd_t * dir;
-
-	if (address >= end)
-		BUG();
-	dir = pgd_offset(mm, address);
-	do {
-		freed += zap_pmd_range(tlb, dir, address, end - address);
-		address = (address + PGDIR_SIZE) & PGDIR_MASK;
-		dir++;
-	} while (address && (address < end));
-
-	/*
-	 * Update rss for the mm_struct (not necessarily current->mm)
-	 * Notice that rss is an unsigned long.
-	 */
-	if (mm->rss > freed)
-		mm->rss -= freed;
-	else
-		mm->rss = 0;
-}
-
 /*
  * remove user pages in a given range.
  */
 void zap_page_range(struct mm_struct *mm, unsigned long address, unsigned long size)
 {
-	unsigned long start = address, end = address + size;
 	mmu_gather_t *tlb;
+	pgd_t * dir;
+	unsigned long start = address, end = address + size;
+	int freed = 0;
+
+	dir = pgd_offset(mm, address);
 
 	/*
 	 * This is a long-lived spinlock. That's fine.
@@ -400,11 +390,29 @@ void zap_page_range(struct mm_struct *mm, unsigned long address, unsigned long s
 		BUG();
 	spin_lock(&mm->page_table_lock);
 	flush_cache_range(mm, address, end);
-
 	tlb = tlb_gather_mmu(mm);
-	unmap_page_range(tlb, mm, address, end);
+
+	do {
+		freed += zap_pmd_range(tlb, dir, address, end - address);
+		address = (address + PGDIR_SIZE) & PGDIR_MASK;
+		dir++;
+	} while (address && (address < end));
+
+	/* this will flush any remaining tlb entries */
 	tlb_finish_mmu(tlb, start, end);
+
+	/*
+	 * Update rss for the mm_struct (not necessarily current->mm)
+	 * Notice that rss is an unsigned long.
+	 */
+	if (mm->rss > freed)
+		mm->rss -= freed;
+	else
+		mm->rss = 0;
 	spin_unlock(&mm->page_table_lock);
+#ifdef __arm__
+	memc_update_mm(mm);
+#endif
 }
 
 /*
@@ -534,6 +542,8 @@ bad_page:
 	goto out;
 }
 
+EXPORT_SYMBOL(get_user_pages);
+
 /*
  * Force in an entire range of pages from the current process's user VA,
  * and pin them in physical memory.  
@@ -592,6 +602,8 @@ int map_user_kiobuf(int rw, struct kiobuf *iobuf, unsigned long va, size_t len)
  * occurs, the number of bytes read into memory may be less than the
  * size of the kiobuf, so we have to stop marking pages dirty once the
  * requested byte count has been reached.
+ *
+ * Must be called from process context - set_page_dirty() takes VFS locks.
  */
 
 void mark_dirty_kiobuf(struct kiobuf *iobuf, int bytes)
@@ -609,7 +621,7 @@ void mark_dirty_kiobuf(struct kiobuf *iobuf, int bytes)
 		page = iobuf->maplist[index];
 		
 		if (!PageReserved(page))
-			SetPageDirty(page);
+			set_page_dirty(page);
 
 		remaining -= (PAGE_SIZE - offset);
 		offset = 0;
@@ -818,6 +830,9 @@ int zeromap_page_range(unsigned long address, unsigned long size, pgprot_t prot)
 	} while (address && (address < end));
 	spin_unlock(&mm->page_table_lock);
 	flush_tlb_range(mm, beg, end);
+#ifdef __arm__
+	memc_update_mm(current->mm);
+#endif
 	return error;
 }
 
@@ -900,6 +915,9 @@ int remap_page_range(unsigned long from, unsigned long phys_addr, unsigned long
 	} while (from && (from < end));
 	spin_unlock(&mm->page_table_lock);
 	flush_tlb_range(mm, beg, end);
+#ifdef __arm__
+	memc_update_mm(current->mm);
+#endif
 	return error;
 }
 
@@ -916,6 +934,9 @@ static inline void establish_pte(struct vm_area_struct * vma, unsigned long addr
 	set_pte(page_table, entry);
 	flush_tlb_page(vma, address);
 	update_mmu_cache(vma, address, entry);
+#ifdef __arm__
+	memc_update_addr(vma->vm_mm, entry, address);
+#endif
 }
 
 /*
@@ -927,6 +948,7 @@ static inline void break_cow(struct vm_area_struct * vma, struct page * new_page
 	flush_page_to_ram(new_page);
 	flush_cache_page(vma, address);
 	establish_pte(vma, address, page_table, pte_mkwrite(pte_mkdirty(mk_pte(new_page, vma->vm_page_prot))));
+	flush_icache_page(vma, new_page);
 }
 
 /*
@@ -1186,6 +1208,9 @@ static int do_swap_page(struct mm_struct * mm,
 
 	/* No need to invalidate - it was non-present before */
 	update_mmu_cache(vma, address, pte);
+#ifdef __arm__
+	memc_update_addr(vma->vm_mm, pte, address);
+#endif
 	spin_unlock(&mm->page_table_lock);
 	return ret;
 }
@@ -1231,6 +1256,9 @@ static int do_anonymous_page(struct mm_struct * mm, struct vm_area_struct * vma,
 
 	/* No need to invalidate - it was non-present before */
 	update_mmu_cache(vma, addr, entry);
+#ifdef __arm__
+	memc_update_addr(vma->vm_mm, *page_table, addr);
+#endif
 	spin_unlock(&mm->page_table_lock);
 	return 1;	/* Minor fault */
 
@@ -1311,6 +1339,9 @@ static int do_no_page(struct mm_struct * mm, struct vm_area_struct * vma,
 
 	/* no need to invalidate: a not-present page shouldn't be cached */
 	update_mmu_cache(vma, address, entry);
+#ifdef __arm__
+	memc_update_addr(vma->vm_mm, *page_table, address);
+#endif
 	spin_unlock(&mm->page_table_lock);
 	return 2;	/* Major fault */
 }
@@ -1494,3 +1525,24 @@ int make_pages_present(unsigned long addr, unsigned long end)
 			len, write, 0, NULL, NULL);
 	return ret == len ? 0 : -1;
 }
+
+struct page * vmalloc_to_page(void * vmalloc_addr)
+{
+	unsigned long addr = (unsigned long) vmalloc_addr;
+	struct page *page = NULL;
+	pmd_t *pmd;
+	pte_t *pte;
+	pgd_t *pgd;
+	
+	pgd = pgd_offset_k(addr);
+	if (!pgd_none(*pgd)) {
+		pmd = pmd_offset(pgd, addr);
+		if (!pmd_none(*pmd)) {
+			pte = pte_offset(pmd, addr);
+			if (pte_present(*pte)) {
+				page = pte_page(*pte);
+			}
+		}
+	}
+	return page;
+}
